//
// distributions.cc
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#include "distributions.h"

namespace ntk
{
  
  // Taken from the Gnu Gama library  http://www.gnu.org/software/gama/ 
  double StudentDistrib::InvCdf::
  operator() (double palfa) const
  {
    ntk_assert(palfa >= 0 && palfa <= 1.0, "Bad palfa.");
    palfa = 1.0-palfa;
    float alfa = palfa;
    double N = p.v;
    if(alfa > 0.5) alfa=1.0-alfa;
    alfa *= 2;

    if(palfa == 0.5) return 0;

    if (N <= 1)
    {
      float a = M_PI/2*alfa;
      float stu_ = cos(a)/sin(a);
      if (palfa > 0.5) stu_ = -stu_;
      return stu_;
    }

    if (N <= 2)
    {
      float stu_ = sqrt(2.0/(alfa*(2.0-alfa))-2.0);
      if (palfa > 0.5) stu_ = -stu_;
      return stu_;
    }

    float r = N;
    float a = 1.0/(r-0.5);
    float b = 48.0/(a*a);
    float c = ((20700.0*a/b-98.0)*a-16.0)*a+96.36;
    float d = ((94.5/(b+c)-3.0)/b+1.0)*sqrt(M_PI/2*a)*r;
    float x = d*alfa, xx = 2.0/r;
    float y = pow(x,xx);

    if (y > a+0.05)
    {
      x = -norminv(0.5*alfa);
      y = x*x;
      if (N < 5) c += 0.3*(r-4.5)*(x+0.6);
      c = (((0.05*d*x-5.0)*x-7.0)*x-2.0)*x+b+c;
      y = (((((0.4*y+6.3)*y+36.0)*y+94.5)/c-y-3.0)/b+1.0)*x;
      y = a*y*y;
      if (y <= 0.002)
        y = 0.5*y*y+y;
      else
        y = exp(y)-1.0;
      float stu_ = sqrt(r*y);
      if (palfa > 0.5) stu_ = -stu_;
      return stu_;
    }

    y = ((1.0/(((r+6.0)/(r*y)-0.089*d-0.822)*(r+2.0)*3.0)+0.5/(r+4.0))
        *y-1.0)*(r+1.0)/(r+2.0)+1.0/y;
    float stu_ = sqrt(r*y);
    if (palfa > 0.5) stu_ = -stu_;
    return stu_;
  }
  
  unsigned draw_from_distrib(const vcl_vector<double>& distrib, vnl_random& rgen)
  {
    double x = rgen.drand64();
    double sum = 0;
    foreach_idx(i, distrib)
      {
        sum += distrib[i];
        if (x <= sum)
          return i;
      }
    return distrib.size()-1;
  }
  
}
