/*
** debug.h
** Login : <nicolas.burrus@ensta.fr>
** Started on  Sun Oct 15 18:25:24 2006 Nicolas Burrus
** $Id$
**
** Copyright (C) 2006, 2007 Nicolas Burrus
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef   	NTK_UTILS_DEBUG_H_
# define   	NTK_UTILS_DEBUG_H_

# include <ntk/core.h>

namespace ntk
{
  extern int ntk_debug_level;
}

class NtkDebug
{
  public:
    ~NtkDebug()
    { Logger::writeToLog(s); }

  public:
    template <class T>
    void print(const T& rhs) const
    { s << rhs; }

    template <class T>
    void print(const T* rhs) const
    {
      s << typeid(rhs).name()
          << "("
          << String::toHexString((int64)(void*)rhs)
          << ")";
    }

    void print(const char* rhs) const
    {
      s << rhs;
    }

    void print(const juce_wchar* rhs) const
    {
      s << rhs;
    }

  private:
    mutable String s;
};

template <class T>
inline const NtkDebug& operator<<(const NtkDebug& d, const T& rhs)
{ d.print(rhs); return d; }

template <class T>
inline const NtkDebug& operator<<(const NtkDebug& d, const T* rhs)
{ d.print(rhs); return d; }

template <class T>
inline const NtkDebug& operator<<(const NtkDebug& d, T* rhs)
{ d.print((const T*) rhs); return d; }

const NtkDebug& operator<<(const NtkDebug& d, const PropertySet& set);

#ifndef NDEBUG
# define ntk_dbg(level) if (level <= ntk::ntk_debug_level) NtkDebug()
#else
# define ntk_dbg(level) if (0) NtkDebug()
#endif // ndef NDEBUG

# define ntk_log() NtkDebug()

#ifndef NDEBUG
# define ntk_dbg_print(value, level) \
  ntk_dbg(level) << #value": " << value;
#else
# define ntk_dbg_print(value, level)
#endif // ndef NDEBUG

namespace ntk {
  
  void assert_failure(const char* where, const char* what, const char* cond);
  void fatal_error(const char* what, int code=1);

}

#ifdef __GNUC__
# define PRETTY_FUNCTION __PRETTY_FUNCTION__
#else
# define PRETTY_FUNCTION ""
#endif // __GNUC__

#ifndef NDEBUG
# define ntk_assert(cond,what) if (!(cond)) ntk::assert_failure(PRETTY_FUNCTION, what, #cond);
#else
# define ntk_assert(cond,what)
#endif // ndef NDEBUG

#define ntk_ensure(cond,what) if (!(cond)) { ntk::assert_failure(PRETTY_FUNCTION, what, #cond); \
                                             ntk::fatal_error(what); }

#define ntk_dbg_enter_function(level) \
  ntk_dbg(level) << "<Entering> " << PRETTY_FUNCTION \
                 << " (" << this << ")" \
                 << " (" << Thread::getCurrentThreadId() << ")";

#define ntk_dbg_leave_function(level) \
  ntk_dbg(level) << "</Leaving> " << PRETTY_FUNCTION \
                 << " (" << this << ")" \
                 << " (" << Thread::getCurrentThreadId() << ")";

#endif 	    /* !NTK_UTILS_DEBUG_H_ */
