/*
** vil_utils.h
** Login : <nicolas.burrus@lrde.epita.fr>
** Started on  Mon Jun  6 21:15:57 2005 Nicolas Burrus
** $Id$
**
** Copyright (C) 2005, 2006, 2007 Nicolas Burrus
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef   	NTK_VXL_VIL_UTILS_H_
# define   	NTK_VXL_VIL_UTILS_H_

# include <ntk/core.h>
# include <ntk/vxl/types.h>

# include <vil/vil_image_view.h>
# include <vil/vil_pixel_traits.h>
# include <vil/vil_convert.h>
# include <vil/algo/vil_structuring_element.h>
# include <vil/vil_load.h>
# include <vil/vil_save.h>

# include <vnl/vnl_random.h>

# define for_all_xy(im) \
for (unsigned y = 0; y < (im).nj(); ++y) \
for (unsigned x = 0; x < (im).ni(); ++x)

# define for_all_xyp(im) \
for (unsigned p = 0; p < (im).nplanes(); ++p) \
for (unsigned y = 0; y < (im).nj(); ++y) \
for (unsigned x = 0; x < (im).ni(); ++x)

namespace ntk {

  struct vil_point
  {
    vil_point() : x(-1), y(-1) {}
    vil_point(short x, short y) : x(x), y(y) {}
    void setXY(short x, short y) { this->x = x; this->y = y; }
    vil_point& operator+= (const vil_point& rhs)
    { x += rhs.x; y += rhs.y; return *this; }
    short x;
    short y;
  };

  inline vil_point operator+(const vil_point& lhs, const vil_point& rhs)
  {
    vil_point p = lhs;
    p += rhs;
    return p;
  }

  const vcl_vector<vil_point>& vil_neighb_c8();

  vil_structuring_element vil_disk(double r);
  const vil_structuring_element& vil_win_c8p();
  const vil_structuring_element& vil_win_c4p();

  template <class T>
  void im_crop_from_center (const vil_image_view<T>& in, vil_image_view<T>& out);

  template <class T>
  void vil_invert(const vil_image_view<T>& src, vil_image_view<T>& dest);
  void vil_invert(const vil_image_view<bool>& src, vil_image_view<bool>& dest);

  bool vil_save_round(const vil_float_image& im, const char* filename);
  bool vil_save_round(const vil_int_image& im, const char* filename);

  bool vil_save_stretch(const vil_float_image& im, const char* filename);
  bool vil_save_stretch(const vil_int_image& im, const char* filename);

  void vil_sobel_intensity(const vil_byte_image& src, vil_byte_image& dest,
                           const vil_border<vil_byte_image>& border);

  template <class T>
  T gradient_magnitude_2x2(const vil_image_view<T>& src, int x, int y, int value);

  template <class T>
  vil_image_view<T> gradient_magnitude_2x2(const vil_image_view<T>& src);

  vil_byte_image ideal_subsample_div2(vil_byte_image& im);
  vil_byte_image ideal_subsample_div4(vil_byte_image& im);
  vil_byte_image well_sampled_noise(unsigned ni, unsigned nj, vnl_random& rgen);
  vil_byte_image well_sampled_noise(unsigned ni, unsigned nj, const vcl_vector<double>& distrib, vnl_random& rgen);

  void juce_to_vil(const Image& image, ntk::vil_byte_image& result);
  Image* vil_to_juce(const ntk::vil_byte_image& vil_im);

  template <class T>
  void
  vil_convert_cast(const vil_image_view_base_sptr& src, vil_image_view<T>& dest)
  {
    vil_image_view_base_sptr tmp = vil_convert_cast(T(), src);
    dest = tmp;
  }

  template <class T>
  bool has_different_neighbor_forward_8conn(const vil_image_view<T>& im,
                                            uint32 x, uint32 y)
  {
    const T& v = im(x,y);
    if (im.in_range(x+1,y) && im(x+1,y) != v) return true;
    if (im.in_range(x,y+1) && im(x,y+1) != v) return true;
    if (im.in_range(x+1,y+1) && im(x+1,y+1) != v) return true;
    if (im.in_range(x+1,y-1) && im(x+1,y-1) != v) return true;
    return false;
  }

} // end of ntk

# include "vil_utils.hxx"

#endif	    /* !NTK_VXL_VIL_UTILS_H_ */
