//
// vil_utils.hxx
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

namespace ntk
{

  template <class T>
  void im_crop_from_center (const vil_image_view<T>& in, 
                            vil_image_view<T>& out)
  {
    assert(in.ni() >= out.ni());
    assert(in.nj() >= out.nj());
    for (int i = 0; i < out.ni(); ++i)
      for (int j = 0; j < out.nj(); ++j)
        out(i,j) = in(i+(in.ni()-out.ni())/2, j+(in.nj()-out.nj())/2);      
  }

  template <class T>
  void vil_invert(const vil_image_view<T>& src, vil_image_view<T>& dest)
  {
    T zero = 0; // avoid warnings.
    if (vil_pixel_traits<T>::minval() < zero)
    {
      dest.clear();
      return;
    }
    dest.set_size(src.ni(), src.nj(), src.nplanes());
    for_all_xyp(src)
      dest(x,y,p) = vil_pixel_traits<T>::maxval() - src(x,y,p);
  }
    
  template <class T>
  T gradient_magnitude_2x2(const vil_image_view<T>& src, 
                           int x, int y, 
                           int value)
  {
    if (x == (int)src.ni()-1 && y == (int)src.nj()-1)
      return vcl_abs(2*value - src(x,y-1) - src(x-1,y))/2;

    if (x == (int)src.ni()-1)
      return vcl_abs(2*value - src(x,y+1) - src(x-1,y))/2;

    if (y == (int)src.nj()-1)
      return vcl_abs(2*value - src(x+1,y) - src(x,y-1))/2;

    return vcl_abs(2*value - src(x+1,y) - src(x,y+1)) / 2;
  }

  template <class T>
  vil_image_view<T> gradient_magnitude_2x2(const vil_image_view<T>& src)
  {
    vil_image_view<T> dst;
    dst.set_size(src.ni(), src.nj());
    for_all_xy(src)
      dst(x,y) = gradient_magnitude_2x2(src, x, y, src(x,y));
    return dst;
  }
  
} // end of ntk

