#include "juce.h"

class JUCEDemoApplication : public JUCEApplication
{
public:
  JUCEDemoApplication(){}
  ~JUCEDemoApplication(){}
  
  void initialise (const String& commandLine)
  {
    Logger::outputDebugString (collectSomeSystemInfo());
    // AlertWindow::showNativeDialogBox("Test Juce", "Just a simple dialog.", false);
    quit();
  }
  
  void shutdown() {}
  
  const String getApplicationName() { return T("JUCE Demo"); }
  const String getApplicationVersion() { return T("1.0"); }
  bool moreThanOneInstanceAllowed() { return true; }
  void anotherInstanceStarted (const String& commandLine) {}

private:
    static const String collectSomeSystemInfo()
    {
        String systemInfo;

        systemInfo
          << T("Time and date: ") << Time::getCurrentTime().toString (true, true)
          << T("\nOperating system: ") << SystemStats::getOperatingSystemName()
          << T("\nCPU vendor: ") << SystemStats::getCpuVendor()
          << T("\nCPU speed: ") << SystemStats::getCpuSpeedInMegaherz() << T("MHz\n")
          << T("\nNumber of CPUs: ") << SystemStats::getNumCpus()
          << T("\nCPU has MMX: ") << (SystemStats::hasMMX() ? T("yes") : T("no"))
          << T("\nCPU has SSE: ") << (SystemStats::hasSSE() ? T("yes") : T("no"))
          << T("\nCPU has SSE2: ") << (SystemStats::hasSSE2() ? T("yes") : T("no"))
          << T("\nCPU has 3DNOW: ") << (SystemStats::has3DNow() ? T("yes") : T("no"))
          << T("\nMemory size: ") << SystemStats::getMemorySizeInMegabytes() << T("MB\n");

        int64 macAddresses[8];
        const int numAddresses = SystemStats::getMACAddresses (macAddresses, 8);

        for (int i = 0; i < numAddresses; ++i)
        {
            systemInfo
              << T("Found network card MAC address: ")
              << String::formatted (T("%02x-%02x-%02x-%02x-%02x-%02x\n"),
                                    0xff & (int) (macAddresses [i] >> 40),
                                    0xff & (int) (macAddresses [i] >> 32),
                                    0xff & (int) (macAddresses [i] >> 24),
                                    0xff & (int) (macAddresses [i] >> 16),
                                    0xff & (int) (macAddresses [i] >> 8),
                                    0xff & (int) macAddresses [i]);
        }

        systemInfo
          << T("Current executable file: ")
          << File::getSpecialLocation (File::currentExecutableFile).getFullPathName()
          << T("\nCurrent application file: ")
          << File::getSpecialLocation (File::currentApplicationFile).getFullPathName()
          << T("\nUser home directory: ")
          << File::getSpecialLocation (File::userHomeDirectory).getFullPathName()
          << T("\nUser documents directory: ")
          << File::getSpecialLocation (File::userDocumentsDirectory).getFullPathName()
          << T("\nUser application data directory: ")
          << File::getSpecialLocation (File::userApplicationDataDirectory).getFullPathName()
          << T("\nCommon application data directory: ")
          << File::getSpecialLocation (File::commonApplicationDataDirectory).getFullPathName()
          << T("\nTemp directory: ")
          << File::getSpecialLocation (File::tempDirectory).getFullPathName()
          << T("\n\n");

        return systemInfo;
    }
};

START_JUCE_APPLICATION (JUCEDemoApplication)
