//
// edison.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2008
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_edison_H
# define ACSEGMENTOR_edison_H

# include <ntk/core.h>
# include <ntk/vxl/vil_utils.h>
# include <ntk/utils/process.h>

# include "segmentor.h"

/**
 *
 * Mean Shift: A Robust Approach toward Feature Space Analysis
 * Dorin Comaniciu and Peter Meer
 * 2004
 * 
 * Relying on the EDISON implementation.
 */

class EdisonFunctor : public SegmentationFunctor
{
  public:
    EdisonFunctor(double hs, double hr, double min_area) 
      : m_hs(hs), m_hr(hr), m_min_area(min_area)
    {}
  
    virtual String name() const 
    { 
      String s = "Edison";
      s << "HS" << m_hs << "HR" << m_hr << "M" << m_min_area;
      return s;
    }
  
    virtual vil_image_view<int32> operator()(const ntk::vil_byte_image& im)    
    {
      String command;
      vil_save(im, "/tmp/edison_input.pgm");
      command << "edison.sh " 
          << " " << m_hs
          << " " << m_hr
          << " " << m_min_area
          << " " << "/tmp/edison_input.pgm" 
          << " " << "/tmp/edison_output.pgm";
      int exit_code = ntk::shell_command(command, true);
      if (exit_code != 0) // FIXME: does not work on mac ?
      {
        ntk_throw_exception("Could not execute edison program.");
      } 
      ntk::vil_byte_image raw_output = vil_load("/tmp/edison_output.pgm");
      if (!raw_output) ntk_throw_exception("Could not run edison.");
      File("/tmp/edison_input.pgm").deleteFile();
      File("/tmp/edison_output.pgm").deleteFile();
      vil_image_view<int32> seg (raw_output.ni(), raw_output.nj());
      for_all_xy(seg) seg(x,y) = raw_output(x,y);
      return seg;
    }
  
  private:
    double m_hs;
    double m_hr;
    double m_min_area;
};

#endif // ndef ACSEGMENTOR_edison_H
