//
// learning_window.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_gui_learning_window_H
# define ACSEGMENTOR_gui_learning_window_H

# include <ntk/core.h>
# include "../ac_config.h"
# include "learning_main_component.h"

class LearningWindow : public DocumentWindow
{
  public:
    LearningWindow(const PropertySetSubject& properties)
        : DocumentWindow("Acsegmentor "AC_VERSION_STRING" - Learning", Colours::grey,
                         DocumentWindow::allButtons, true),
        m_properties(properties)
    {
      // setUsingNativeTitleBar(true);
      setResizable(true, false);
      setSize(1200, 500);
      setCentreRelative(0.4f, 0.4f);

      m_main_component = new LearningMainComponent(m_properties);
      m_main_component->setVisible(true);
      setContentComponent(m_main_component);
    }

    virtual ~LearningWindow()
    {}

  public:
    void startLearning()
    {
      if (!m_main_component->startLearning())
        delete this;
    }

  protected:
    virtual void closeButtonPressed()
    {
      m_main_component->askForQuit();
      delete this;
    }

  private:
    PropertySetSubject m_properties;
    LearningMainComponent* m_main_component;
};

#endif // ndef ACSEGMENTOR_gui_learning_window_H
