//
// srm.h
//
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2008
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_srm_H
# define ACSEGMENTOR_srm_H

# include <ntk/core.h>
# include <ntk/vxl/vil_utils.h>
# include <ntk/utils/process.h>

# include "segmentor.h"

/**
 *
 * Statistical Region Merging
 * Richard Nock and Frank Nielsen
 * PAMI 2004
 * 
 * Relying on their binary.
 */

class SrmFunctor : public SegmentationFunctor
{
  public:
    SrmFunctor(double complexity) : m_complexity(complexity)
    {}
  
    virtual String name() const 
    { 
      String s = "Srm";
      s << "Q" << m_complexity;
      return s;
    }
  
    virtual vil_image_view<int32> operator()(const ntk::vil_byte_image& im)    
    {
      String command;
      vil_save(im, "/tmp/srm_input.pgm");
      command << "srm.sh" << " " << m_complexity << " " << "/tmp/srm_input.pgm" << " " << "/tmp/srm_output.pgm";
      int exit_code = ntk::shell_command(command, true);
      if (exit_code != 0) // FIXME: does not work on mac ?
      {
        ntk_throw_exception("Could not execute srm program.");
      } 
      ntk::vil_byte_image raw_output = vil_load("/tmp/srm_output.pgm");
      if (!raw_output) ntk_throw_exception("Could not run srm.");
      File("/tmp/srm_input.pgm").deleteFile();
      File("/tmp/srm_output.pgm").deleteFile();
      vil_image_view<int32> seg (raw_output.ni(), raw_output.nj());
      for_all_xy(seg) seg(x,y) = raw_output(x,y);
      return seg;
    }
  
  private:
    double m_complexity;
};

#endif // ndef ACSEGMENTOR_srm_H
