//
// watershed.h
// 
// Author: Nicolas Burrus <nicolas.burrus@ensta.fr>, (C) 2007
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//

#ifndef ACSEGMENTOR_watershed_H
# define ACSEGMENTOR_watershed_H

# include <ntk/ntk.h>
# include "segmentor.h"

/**
 * Watershed by Vincent & Soille
 * Copy/Paste implementation from Olena http://olena.lrde.epita.fr
 */
vil_image_view<int32> watershed_segment(const ntk::vil_byte_image& im, 
                                        double sigma = 0.5);

class WatershedFunctor : public SegmentationFunctor
{
  public:
    WatershedFunctor(double sigma)
      : m_sigma(sigma)
    {}
  
    virtual String name() const 
    { 
      String s = "Watershed";
      s << "S" << m_sigma;
      return s;
    }
  
    virtual vil_image_view<int32> operator()(const ntk::vil_byte_image& im)
    { return watershed_segment(im, m_sigma); }
  
  private:
    double m_sigma;
};

#endif // ndef ACSEGMENTOR_watershed_H
